# 如何添加新的API

随着业务的增长，我们现有的 API 可能不满足需求，那么，怎样去新增一些 API 呢？本文以“分享”这个业务场景为例，给出一些说明。

## 一个业务需要包含哪些东西？

在 Teambition SDK 中，一个单独的业务需要包含这么几种东西：

- Schema
- Fetch API
- Model
- Reactive API
- WebSocket 的对应处理
- 测试

## Schema

Schema 是一个业务中所出现的数据类型，比如说，任务里面有 TaskSchema，子任务里面有 SubtaskSchema，它是对前后端通信的一个数据格式约束，也作为 SDK 中数据格式检查、存储的依据，还是视图层与 SDK 通信的桥梁。

所以，Schema 就是一个数据格式的描述，在里面定义清楚数据类型即可。

## Fetch API

Fetch API 是对 HTTP 请求的简单封装，这层东西很容易写，大致就是这么几个步骤：

- 编写一个从 BaseFetch 派生的类
- 列举本业务相关的各类 HTTP 请求接口
- 使用 interface，定义这些请求接口的参数格式
- 挨个实现对应的请求接口封装，每个封装成一个方法，返回类型为 Promise

这一层封装完成之后，其实已经可以给外部使用了，只是它们用的是最原始的接口，并不具有 Reactive 特性

## Model

要做好模型层的封装，需要理解两个类：BaseModel，BaseCollection。

BaseModel 是某个模型的基础类，如果要实现一个业务模型，比如 PostModel，首先要继承 BaseModel，在 BaseModel 中，隐含了对 BaseCollection 的引用。

怎么理解 BaseModel 和 BaseCollection 的关系呢？

在 BaseModel上，实际承担的两个职责：

- 对于某一种业务模型的包装和处理
- 业务模型对外的接口

而 BaseCollection 的职责只有一种：

- 对某种业务类型的批量接口的包装和处理

它不直接提供对外接口，外部要跟 Collection 打交道，必须从 BaseModel 那里入手。例如：

```
PostModel::getPosts
```

这么一个方法，其实是 PostModel 的批量接口，但它仍然是通过 PostModel 这个派生于 BaseModel 的模型暴露出来，而不是通过一个叫做 PostCollection 的类。而在 getPosts 方法的内部实现中，还是会去使用 Collection 的。

具体到某种业务的模型，在实现的时候，需要注意这么一些方面：

- 模型的基本增删查改，都不需要额外的考虑，直接调用 BaseModel 里面定义的即可
- 批量操作需要做一些额外的事情：
  - 指定存储索引。为什么要指定存储索引呢？因为同一个数据模型，可能分别被关联在多个不同的集合中，必须通过某种方式，把它们区分开，不然就会混乱。
  - 作一些业务上的判断，比如数据为空的处理，分页的处理等等

## Reactive API

这一步是对上层提供 Reactive 的 API，主要要考虑几个东西：

这个 API 是冷的还是热的（这两个名词参考 RxJS 对流的定义：[Hot VS Cold Observables](https://medium.com/@benlesh/hot-vs-cold-observables-f8094ed53339)，[Demystifying Cold and Hot Observables in RxJS](https://egghead.io/lessons/rxjs-demystifying-cold-and-hot-observables-in-rxjs)）？一般来说，查询获取是 Hot Observable，因为需要持续通知变动，而删除和更新是 Cold Observable，只需通知一次即可终止。

然后根据这个API的冷热情况，确定不同的返回方式，冷的需要手动 complete 掉。

这些 API 整体是与 Fetch API 对应的，只是实现不同。这样，Reactive API 就写好了。

到这里，整个 API 层其实就写好了，但是，作为开发者，看到这里，肯定会觉得疑惑，因为直到现在，并未涉及关联数据的更新，比如：

- 更新某个 task 导致其它对此 task 订阅的变更
- 更新某个 task 导致其所属项目的 tasks 的变更
- 更新某个 task 导致其所属 project 的变更

这些事情是在哪里做呢？

其实这些事情基本不需要开发者自己去做，它们会在 BaseModel 提供的一些封装里面做掉，比如这个类提供的 add，update 等方法，这方面的解释不在本文中提及。

## WebSocket 的处理

这部分要做的事情其实很简单，因为服务端提供的 WebSocket 的推送，其实是有一定规则的，现有的 SDK 核心层其实已经做了这方面的处理。当有新业务出现的时候，只需在 EventMaps.ts 这个文件中，往 typeMap 里添加对应的类型定义就可以了。

## 测试

参见《Teambition SDK 的测试》一文
